﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DatabaseManipulation
{

   public  enum DatabaseOperation
    {
        Load,
        Edit,
        Delete,
        Save
    }

    public class DatabaseClass
    {
        private List<Product> productList = new List<Product>();
        private List<Employee> employeeList = new List<Employee>();
        private List<Store> storeList = new List<Store>();
        private int numberOfStore = 0;
        private string comboString = "";
        private string productlistJson = "";
        private string employeeListJson = "";
        private string storeListJson = "";
        private string productStoreJson = "";

        public int NumberOfStore
        {
            get
            {
                return numberOfStore;
            }
        }

        public string ComboString
        {
            get
            {
                return comboString;
            }
        }

        public string ProductlistJson
        {
            get
            {
                return productlistJson;
            }
        }

        public string EmployeeListJson
        {
            get
            {
                return employeeListJson;
            }
        }

        public string StoreListJson
        {
            get
            {
                return storeListJson;
            }
        }

        public string ProductStoreJson
        {
            get
            {
                return productStoreJson;
            }
        }

        public void LoadFromDatabase(DatabaseOperation op)
        {
            try
            {
                using (var session = NHibernateHelper.OpenSession())
                {
                    using (var tx = session.BeginTransaction())
                    {
                        productList = (List<Product>)session.QueryOver<Product>().List<Product>();
                        employeeList = (List<Employee>)session.QueryOver<Employee>().List<Employee>();
                        storeList = (List<Store>)session.QueryOver<Store>().List<Store>();

                        numberOfStore = storeList.Count;

                        var productResult = from p in productList
                                            select new { prdid = p.Id, prdname = p.Name, prdprice = p.Price };
                        productlistJson = JsonConvert.SerializeObject(productResult);

                        var employeeResult = from p in employeeList
                                             select new { emplid = p.Id, emplfirstname = p.FirstName, empllastname = p.LastName, emplstoreid = p.Store.Id };
                        employeeListJson = JsonConvert.SerializeObject(employeeResult);

                        var storeResult = from p in storeList
                                          select new { strid = p.Id, strname = p.Name };
                        storeListJson = JsonConvert.SerializeObject(storeResult);

                        var stroreproductdata = from p in productList
                                                select new { productid = p.Id, storeid = p.StoresStockedIn[0].Id };
                        productStoreJson = JsonConvert.SerializeObject(stroreproductdata);

                        List<string> items = new List<string>();
                        for (int i = 0; i < numberOfStore; i++)
                        {
                            items.Add((i + 1).ToString());
                        }
                        comboString = JsonConvert.SerializeObject(items);
                        tx.Commit();
                    }
                }
            }
            catch (Exception ex)
            {
            }
        }

        public void EditInDatabase(string aFullText, DatabaseOperation op)
        {
            string[] authorsList = aFullText.Split(':');
            string firstname = authorsList[0];
            string lastname = authorsList[1];
            string productname = authorsList[2];

            double productPrice = 0;
            try
            {
                productPrice = double.Parse(authorsList[3], CultureInfo.InvariantCulture);
            }
            catch (Exception) { }
            string selectedid = authorsList[4];
            string storename = authorsList[5];

            using (var session = NHibernateHelper.OpenSession())
            {
                using (var transaction = session.BeginTransaction())
                {
                    bool found = false;
                    Store store = null;
                    Store newstore = new Store { Name = storename };

                    foreach (var item in storeList)
                    {
                        if (item.Name.Contains(storename) == true)
                        {
                            found = true;
                            store = item;
                            break;
                        }
                    }

                    var paramName = new Employee { FirstName = firstname, LastName = lastname };
                    var paramProduct = new Product { Name = productname, Price = productPrice };

                    if (found == true)
                    {
                        AddEmployeesToStore(store, paramName);
                        AddProductsToStore(store, paramProduct);
                        session.SaveOrUpdate(store);
                    }
                    else
                    {
                        AddEmployeesToStore(newstore, paramName);
                        AddProductsToStore(newstore, paramProduct);
                        session.SaveOrUpdate(newstore);
                    }
                    transaction.Commit();
                }
            }
        }

        public void DeleteFromDatabase(string aFullText, DatabaseOperation op)
        {
            string[] authorsList = aFullText.Split(':');
            string firstname = authorsList[0];
            string lastname = authorsList[1];
            string productname = authorsList[2];

            double productPrice = 0;
            try
            {
                productPrice = double.Parse(authorsList[3], CultureInfo.InvariantCulture);
            }
            catch (Exception) { }
            string storename = authorsList[4];

            using (var session = NHibernateHelper.OpenSession())
            {
                using (var transaction = session.BeginTransaction())
                {
                    bool found = false;
                    Store store = null;
                    Store newstore = new Store { Name = storename };

                    foreach (var item in storeList)
                    {
                        if (item.Name.Contains(storename) == true)
                        {
                            found = true;
                            store = item;
                            break;
                        }
                    }

                    var paramName = new Employee { FirstName = firstname, LastName = lastname };
                    var paramProduct = new Product { Name = productname, Price = productPrice };

                    if (found == true)
                    {
                        AddEmployeesToStore(store, paramName);
                        AddProductsToStore(store, paramProduct);
                        session.SaveOrUpdate(store);
                    }
                    else
                    {
                        AddEmployeesToStore(newstore, paramName);
                        AddProductsToStore(newstore, paramProduct);
                        session.SaveOrUpdate(newstore);
                    }
                    transaction.Commit();
                }
            }
        }

        public void SaveToDatabase(string aFullText, DatabaseOperation op)
        {
            string[] authorsList = aFullText.Split(':');
            string firstname = authorsList[0];
            string lastname = authorsList[1];
            string productname = authorsList[2];

            double productPrice = 0;
            try
            {
                productPrice = double.Parse(authorsList[3], CultureInfo.InvariantCulture);
            }
            catch (Exception) { }
            string storename = authorsList[4];

            using (var session = NHibernateHelper.OpenSession())
            {
                using (var transaction = session.BeginTransaction())
                {
                    bool found = false;
                    Store store = null;
                    Store newstore = new Store { Name = storename };

                    foreach (var item in storeList)
                    {
                        if (item.Name.Contains(storename) == true)
                        {
                            found = true;
                            store = item;
                            break;
                        }
                    }

                    var paramName = new Employee { FirstName = firstname, LastName = lastname };
                    var paramProduct = new Product { Name = productname, Price = productPrice };

                    if (found == true)
                    {
                        AddEmployeesToStore(store, paramName);
                        AddProductsToStore(store, paramProduct);
                        session.SaveOrUpdate(store);
                    }
                    else
                    {
                        AddEmployeesToStore(newstore, paramName);
                        AddProductsToStore(newstore, paramProduct);
                        session.SaveOrUpdate(newstore);
                    }
                    transaction.Commit();
                }
            }
        }

        private void AddProductsToStore(Store store, params Product[] products)
        {
            foreach (var product in products)
            {
                store.AddProduct(product);
            }
        }

        private void AddEmployeesToStore(Store store, params Employee[] employees)
        {
            foreach (var employee in employees)
            {
                store.AddEmployee(employee);
            }
        }

        public void LoadData(int id)
        {
            using (var session = NHibernateHelper.OpenSession())
            {
                using (var tx = session.BeginTransaction())
                {

                    try
                    {

                        var list = session.QueryOver<Product>().List();
                        var list1 = session.QueryOver<Employee>().List();
                        var list2 = session.QueryOver<Store>().List();

                        //Need work on this unfinished
                    }
                    catch (Exception ex)
                    {
                    }
                }
            }
        }
    }
}
